"use strict";
/*
 * Copyright (c) 2023 Eclipse Foundation, Inc.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * Contributors:
 *   Olivier Goulet <olivier.goulet@eclipse-foundation.org>
 *
 * SPDX-License-Identifier: EPL-2.0
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInterestGroupParticipatingOrganizations = exports.getInterestGroups = exports.getInterestGroup = void 0;
require("isomorphic-fetch");
const eclipsefdn_membership_1 = require("./eclipsefdn.membership");
const apiBasePath = 'https://projects.eclipse.org/api/interest-groups';
const interestGroupMapper = ig => {
    return Object.assign({ leads: ig.leads.map(lead => (Object.assign({ fullName: lead.full_name }, lead))), participants: ig.participants.map(participant => (Object.assign({ fullName: participant.full_name }, participant))), foundationDbProjectId: ig.foundationdb_project_id, gitlab: {
            projectGroup: ig.gitlab.project_group,
            ignoredSubGroups: ig.gitlab.ignored_sub_groups,
        } }, ig);
};
const getInterestGroup = ({ interestGroupNodeId, interestGroupId }) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        let url;
        if (interestGroupId) {
            url = `${apiBasePath}?project_id=foundation-internal.ig.${interestGroupId}`;
        }
        else if (interestGroupNodeId) {
            url = `${apiBasePath}/${interestGroupNodeId}`;
        }
        else {
            throw new TypeError('No interestGroupId or interestGroupNodeId provided to getInterestGroup');
        }
        const response = yield fetch(url);
        if (!response.ok)
            throw new Error(interestGroupId
                ? `Could not fetch interest group for id "${interestGroupId}". Ensure that you are using the same value as project_short_id from the API`
                : `Could not fetch interest group for node id "${interestGroupNodeId}"`);
        // Request without node id will return an array of one element
        const data = interestGroupNodeId
            ? yield response.json()
            : (yield response.json())[0];
        const interestGroup = interestGroupMapper(data);
        return [interestGroup, null];
    }
    catch (error) {
        return [null, error];
    }
});
exports.getInterestGroup = getInterestGroup;
const getInterestGroups = () => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const response = yield fetch(`${apiBasePath}`);
        if (!response.ok)
            throw new Error(`Could not fetch interest groups`);
        const data = yield response.json();
        const interestGroups = data.map(interestGroupMapper);
        return [interestGroups, null];
    }
    catch (error) {
        return [null, error];
    }
});
exports.getInterestGroups = getInterestGroups;
const getInterestGroupParticipatingOrganizations = (options) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const [interestGroup, error] = yield (0, exports.getInterestGroup)(options);
        if (error)
            throw new Error(error);
        /*
          Create an array from a set of organizations from participants and leads. The set guarantees that the
          organizations have no duplicates.
        */
        const participatingOrganizationIds = [
            ...new Set([...interestGroup.participants, ...interestGroup.leads].map(p => p.organization.id))
        ];
        /*
          Get all participating organizations from organization id. Filter out the ones which had errors
          for whatever reason.
        */
        const participatingOrganizations = (yield Promise.all(participatingOrganizationIds.map((participantId) => __awaiter(void 0, void 0, void 0, function* () {
            const [participant, participantError] = yield (0, eclipsefdn_membership_1.getOrganizationById)(participantId);
            if (participantError) {
                console.error(`Could not fetch participant organization from id ${participantId}`);
            }
            return participant;
        })))).filter(participant => participant !== null);
        return [participatingOrganizations, null];
    }
    catch (error) {
        console.error(error);
        return [null, error];
    }
});
exports.getInterestGroupParticipatingOrganizations = getInterestGroupParticipatingOrganizations;
