"use strict";
/*!
 * Copyright (c) 2021, 2022, 2023 Eclipse Foundation, Inc.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v. 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * Contributors:
 *   Zhou Fang <zhou.fang@eclipse-foundation.org>
 *   Eric Poirier <eric.poirier@eclipse-foundation.org>
 *   Olivier Goulet <olivier.goulet@eclipse-foundation.org>
 *
 * SPDX-License-Identifier: EPL-2.0
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.convertToQueryString = exports.validateURL = exports.getNextPage = exports.waitForRender = exports.scrollToAnchor = exports.displayErrMsg = void 0;
// UI utilities
const displayErrMsg = (element, err = '', errText = 'Sorry, something went wrong, please try again later.') => (element.innerHTML = `<div class="alert alert-danger" role="alert"><p><strong>Error ${err}</strong></p> <p>${errText}</p></div>`);
exports.displayErrMsg = displayErrMsg;
const scrollToAnchor = () => {
    const elementId = location.hash.replace('#', '');
    const element = document.getElementById(elementId);
    element.scrollIntoView();
};
exports.scrollToAnchor = scrollToAnchor;
/** Resolves when the given element has finished rendering its children.
  * @param {HTMLElement} element
  * @returns {Promise<undefined>}
  */
const waitForRender = element => new Promise(resolve => {
    // If the element already has children, it's already rendered
    if (element.children.length > 0) {
        resolve();
        return;
    }
    // Otherwise, wait for the element to have children
    const observer = new MutationObserver((mutationList, observer) => {
        for (const mutation of mutationList) {
            if (mutation.type === 'childList') {
                observer.disconnect();
                resolve();
            }
        }
    });
    observer.observe(element, { childList: true });
});
exports.waitForRender = waitForRender;
// API utilities
const getNextPage = (linkHeader) => {
    if (!linkHeader)
        return;
    const links = linkHeader.split(', ');
    const nextLink = links.find(link => link.includes('rel="next"'));
    if (!nextLink)
        return;
    const nextUrl = nextLink.match(/<(.+)>/)[1];
    return nextUrl;
};
exports.getNextPage = getNextPage;
const absUrlPattern = /^[a-zA-Z]+:\/\//;
const validateURL = (url) => {
    if (!url)
        return false;
    const isAbsolute = url.match(absUrlPattern);
    const base = isAbsolute ? undefined : window.location.href;
    let isValidate;
    try {
        // This will fail if the url is not valid
        new URL(url, base);
        isValidate = true;
    }
    catch (error) {
        isValidate = false;
    }
    return isValidate;
};
exports.validateURL = validateURL;
const querystring_1 = __importDefault(require("querystring"));
const convertToQueryString = params => {
    // Filters out undefined params
    const filteredParams = Object.fromEntries(Object
        .entries(params)
        .filter(([_, v]) => v !== undefined));
    return querystring_1.default.stringify(filteredParams);
};
exports.convertToQueryString = convertToQueryString;
